@@ -37,29 +37,47 @@ on each platform. In addition Unicode characters are expanded to their
 Unicode code to work around different font handling on platforms.
 """
 
+# messes up loaded files if set
+# from __future__ import division
+from __future__ import print_function
 import glob
+import os
 import os.path
 import sys
+import subprocess
 
+from veusz.compat import cexec, cstr
 import veusz.qtall as qt4
-import veusz.utils.textrender
+import veusz.utils as utils
 import veusz.document as document
 import veusz.setting as setting
-import veusz.windows.mainwindow
+import veusz.dataimport
 import veusz.document.svg_export as svg_export
 
+# required to get structures initialised
+import veusz.windows.mainwindow
+
+try:
+    import h5py
+except ImportError:
+    h5py = None
+
+try:
+    from astropy.io import fits as pyfits
+except ImportError:
+    try:
+        import pyfits
+    except ImportError:
+        pyfits = None
+
 # these tests fail for some reason which haven't been debugged
 # it appears the failures aren't important however
 excluded_tests = set([
-
-        # fails on Windows
-        'histo.vsz',      # duplicate in long list of values
-        'spectrum.vsz',   # angstrom is split into two on linux
-
-        # fails on Mac OS X
-        'histo.vsz',      # somewhere in long list of values
-        'spectrum.vsz',   # symbol issue
-        'labels.vsz'      # symbol issue
+        # fails on Linux Arm
+        'spectrum.vsz',
+        'hatching.vsz',
+        # don't expect this to work
+        'mathml.vsz',
     ])
 
 class StupidFontMetrics(object):
@@ -87,14 +105,21 @@ class StupidFontMetrics(object):
     def boundingRect(self, c):
         return qt4.QRectF(0, 0, self.height()*0.5, self.height())
 
-_pt = veusz.utils.textrender.PartText
+    def boundingRectChar(self, c):
+        return qt4.QRectF(0, 0, self.height()*0.5, self.height())
+
+_pt = utils.textrender.PartText
 class PartTextAscii(_pt):
     """Text renderer which converts text to ascii."""
     def __init__(self, text):
-        text = unicode(text).encode('ascii', 'xmlcharrefreplace')
+        text = text.encode('ascii', 'xmlcharrefreplace').decode('ascii')
         _pt.__init__(self, text)
+    def render(self, state):
+        _pt.render(self, state)
+    def addText(self, text):
+        self.text += text.encode('ascii', 'xmlcharrefreplace').decode('ascii')
 
-def renderTest(invsz, outfile):
+def renderVszTest(invsz, outfile):
     """Render vsz document to create outfile."""
 
     d = document.Document()
@@ -107,11 +132,16 @@ def renderTest(invsz, outfile):
     for cmd in document.CommandInterface.unsafe_commands:
         cmds[cmd] = getattr(ifc, cmd)
 
-    exec "from numpy import *" in cmds
+    cexec("from numpy import *", cmds)
     ifc.AddImportPath( os.path.dirname(invsz) )
-    exec open(invsz) in cmds
+    cexec(compile(open(invsz).read(), invsz, 'exec'), cmds)
     ifc.Export(outfile)
 
+def renderPyTest(inpy, outfile):
+    """Render py embedded script to create outfile."""
+
+    retn = subprocess.call([sys.executable, inpy, outfile])
+    return retn == 0
 
 class Dirs(object):
     """Directories and files object."""
@@ -124,72 +154,113 @@ class Dirs(object):
         files = ( glob.glob( os.path.join(self.exampledir, '*.vsz') ) +
                   glob.glob( os.path.join(self.testdir, '*.vsz') ) )
 
-        self.invszfiles = [ f for f in files if
-                            os.path.basename(f) not in excluded_tests ]
+        self.infiles = [ f for f in files if
+                         os.path.basename(f) not in excluded_tests ]
+        self.infiles += glob.glob(os.path.join(self.testdir, '*.py'))
 
 def renderAllTests():
     """Check documents produce same output as in comparison directory."""
 
-    print "Regenerating all test output"
+    print("Regenerating all test output")
 
     d = Dirs()
-    for vsz in d.invszfiles:
-        base = os.path.basename(vsz)
-        print base
+    for infile in d.infiles:
+        base = os.path.basename(infile)
+        print(base)
         outfile = os.path.join(d.comparisondir, base + '.selftest')
-        renderTest(vsz, outfile)
+        ext = os.path.splitext(base)[1]
+        if ext == '.vsz':
+            renderVszTest(infile, outfile)
+        elif ext == '.py':
+            renderPyTest(infile, outfile)
 
 def runTests():
-    print "Testing output"
+    print("Testing output")
 
     fails = 0
     passes = 0
+    skipped = 0
 
     d = Dirs()
-    for vsz in sorted(d.invszfiles):
-        base = os.path.basename(vsz)
-        print base
+    for infile in sorted(d.infiles):
+        base = os.path.basename(infile)
+        print(base)
+
+        if ( (base[:5] == 'hdf5_' and h5py is None) or
+             (base[:5] == 'fits_' and pyfits is None) ):
+            print(" SKIPPED")
+            skipped += 1
+            continue
 
         outfile = os.path.join(d.thisdir, base + '.temp.selftest')
-        renderTest(vsz, outfile)
+
+        ext = os.path.splitext(infile)[1]
+        if ext == '.vsz':
+            renderVszTest(infile, outfile)
+        elif ext == '.py':
+            if not renderPyTest(infile, outfile):
+                print(" FAIL: did not execute cleanly")
+                fails += 1
+                continue
+        else:
+            raise RuntimeError('Invalid input file')
 
         comparfile = os.path.join(d.thisdir, 'comparison', base + '.selftest')
 
-        t1 = open(outfile, 'rU').read()
-        t2 = open(comparfile, 'rU').read()
-        if t1 != t2:
-            print " FAIL: results differed"
+        f1 = open(outfile, 'rU')
+        f2 = open(comparfile, 'rU')
+        comp = f1.read() == f2.read()
+        f1.close()
+        f2.close()
+
+        if not comp:
+            print(" FAIL: results differed")
             fails += 1
         else:
-            print " PASS"
+            print(" PASS")
             passes += 1
             os.unlink(outfile)
 
-    print
+    print()
+    if skipped != 0:
+        print('Skipped %i tests' % skipped)
     if fails == 0:
-        print "All tests %i/%i PASSED" % (passes, passes)
+        print("All tests %i/%i PASSED" % (passes, passes))
         sys.exit(0)
     else:
-        print "%i/%i tests FAILED" % (fails, passes+fails)
+        print("%i/%i tests FAILED" % (fails, passes+fails))
         sys.exit(fails)
 
+oldflt = svg_export.fltStr
+def fltStr(v, prec=1):
+    # this is to get consistent rounding to get the self test correct... yuck
+    # decimal would work, but that drags in loads of code
+    # convert float to string with prec decimal places
+
+    v = round(v, prec+2)
+
+    return oldflt(v, prec=prec)
+
 if __name__ == '__main__':
+    os.environ['LC_ALL'] = 'C'
+
     app = qt4.QApplication([])
 
-    veusz.setting.transient_settings['unsafe_mode'] = True
+    setting.transient_settings['unsafe_mode'] = True
 
     # hack metrics object to always return same metrics
     # and replace text renderer with one that encodes unicode symbols
-    veusz.utils.textrender.FontMetrics = StupidFontMetrics
-    veusz.utils.FontMetrics = StupidFontMetrics
-    veusz.utils.textrender.PartText = PartTextAscii
+    utils.textrender.FontMetrics = StupidFontMetrics
+    utils.FontMetrics = StupidFontMetrics
+    utils.textrender.PartText = PartTextAscii
 
     # nasty hack to remove underlining
-    del veusz.utils.textrender.part_commands[r'\underline']
+    del utils.textrender.part_commands[r'\underline']
 
     # dpi (use old values)
     svg_export.dpi = 90.
     svg_export.scale = 1.
+    svg_export.fltStr = fltStr
 
     if len(sys.argv) == 1:
         runTests()
